/******************************************************************************/
/*!
 * @file   ex_common.h
 * @brief
 * @note   スレッド関連の共通定義
 * @date   作成日　2021年12月16日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/

#ifndef _excommon_H_
#define _excommon_H_

#include <zephyr.h>
#include <time.h>

/************************************************************************/
/*																		*/
/* ８３０アプリケーション共通定義（ここから）						   */
/*																		*/
/************************************************************************/
#define DCOM_APP_VERSION		"2.0.1"	// アプリバージョン
#define DCOM_TM_YEAR			123		// 1900年からの経過年数 例：) 2023年 = 123

#define DCOM_RTT_MODE			CONFIG_LOG	// RTT出力 			0：出力しない／1：出力する
#define DCOM_AGPS_OFFLINE		1			// A-GPSオフライン  0：オンライン／1：オフライン

/*======================================================================*/
/* !!!!!!!!!!!!!!!!!!!!!!!!!  CAUTION  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	*/
/* im830/os/http.cのL15にも同define定義がありますが連動できていません	*/
/* もし下記のDCOM_STAGINGの値を変更した場合はhttp.cのDCOM_STAGINGも		*/
/* 同じ値に変更してください												*/
#define DCOM_STAGING			1		// サーバー動作環境 0：本番環境／1：ステージング
/*======================================================================*/

#define DCOM_CPULOAD_KIBANA		1		// CPU使用率kibana送信	0：送信しない／1：送信する
#define DCOM_MODEM_FOTA			0		// モデムFOTA機能 0：無効／1：有効

// 以下は設定連動に移行済み
//#define DCOM_BUZZER			1		// ブザー鳴動       0：鳴動しない／1：鳴動する

#define DCOM_MLOCA_HEALTHCHK	1		// モバロケスレッド死活監視   0：監視しない／1：監視する	// ADD Y.Saisho 2023/5/22 モバロケスレッドの死活監視追加

#define DCOM_TTFF_LOG			0		// TTFF計測用ログのコンパイルスイッチ 0：無効／1：有効		// ADD M.Taguchi 2023/5/16 830001-146 :trunk環境のTTFFが遅い

#define DCOM_SEM_TIMEOUT_MSEC	300000	// セマフォ取得のタイムアウトミリ秒	// ADD K.Saho 2023/05/23 【暫定対処】セマフォ取得の無限待ちを廃止して5分に変更
#define SSL_EXPIRE_DATE 	"2023/05/29 10:00"
/************************************************************************/
/*																		*/
/* ８３０アプリケーション共通定義（ここまで）						   */
/*																		*/
/************************************************************************/

#include "ex_logger_thread.h"
#include "ex_logger.h"
#include "ex_update_thread.h"

/************************************************************************/
/* kibanaのイベントID定義												*/
/*																		*/
/* 書式：xxyynnnn（16進）												*/
/*       xx   レベル DEBUG:0x01、INFO:0x02、WARNING:0xFE、ERROR:0xFF	*/
/*       yy   FB番号 0x01～0x13											*/
/*       nnnn 各FBで自由に付与											*/
/************************************************************************/

/********************************************/
/* xx：エラーレベル							*/
/* 注）0x8nはLTE開局前の保存ログになります	*/
/********************************************/
#define DCOM_LEV_DBG 0x01 // LEVEL：DEBUG
#define DCOM_LEV_INF 0x02 // LEVEL：INFO
#define DCOM_LEV_FRC 0x0D // LEVEL: FORCE
#define DCOM_LEV_WRN 0x0E // LEVEL：WARNING
#define DCOM_LEV_ERR 0x0F // LEVEL：ERROR

/********************************************/
/* yy：FB番号								*/
/********************************************/
#define DCOM_FBNO_MAIN 0x01 // FB：main
#define DCOM_FBNO_BUZZ 0x02 // FB：thread/buzzer
#define DCOM_FBNO_GNSS 0x03 // FB：thread/gnss
#define DCOM_FBNO_LOGG 0x04 // FB：thread/logger
#define DCOM_FBNO_LTEM 0x05 // FB：thread/lte
#define DCOM_FBNO_MLOC 0x06 // FB：thread/mloca
#define DCOM_FBNO_SENS 0x07 // FB：thread/sensor
#define DCOM_FBNO_USIF 0x08 // FB：thread/ui
#define DCOM_FBNO_UPDT 0x09 // FB：thread/update
#define DCOM_FBNO_WDTM 0x0A // FB：thread/wdt
#define DCOM_FBNO_DBUZ 0x0B // FB：device/buzzer
#define DCOM_FBNO_DCAR 0x0C // FB：device/carrier_certs
#define DCOM_FBNO_DFLS 0x0D // FB：device/flash
#define DCOM_FBNO_DGNS 0x0E // FB：device/gnss
#define DCOM_FBNO_DGPI 0x0F // FB：device/gpio
#define DCOM_FBNO_DIMU 0x10 // FB：device/imu
#define DCOM_FBNO_DLTE 0x11 // FB：device/lte
#define DCOM_FBNO_DSER 0x12 // FB：device/serial
#define DCOM_FBNO_DTMP 0x13 // FB：device/temp_humi
#define DCOM_FBNO_DUAR 0x14 // FB：device/uart

/********************************************/
/* kibana送信用マクロ定義					*/
/********************************************/
#define KIBANA_INFO(func, id, cause) 	Supdm_send_kibana(DCOM_LEV_INF, func, id, cause);
#define KIBANA_DEBUG(func, id, cause) 	Supdm_send_kibana(DCOM_LEV_DBG, func, id, cause);
#define KIBANA_WARN(func, id, cause) 	Supdm_send_kibana(DCOM_LEV_WRN, func, id, cause);
#define KIBANA_ERROR(func, id, cause) 	Supdm_send_kibana(DCOM_LEV_ERR, func, id, cause);
#define KIBANA_FORCE(func, id, cause)	Supdm_send_kibana(DCOM_LEV_FRC, func, id, cause);

// ログをflash書き込みレベルごとに下記のマクロを設定していください
// デフォルトは、ERROR、WARNING 残すようにしています。
// DBGはログサイズが多くて、キューが持たないため、1にはしないこと
// →これは別途改善案を検討する
// CHG-S Y.Hirano 2022/3/25 830POC-49 :GNSS測位時間改善

/********************************************************/
/* !!!!!!!!!!!!!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!! */
/* DCOM_RTT_MODEを1にした場合はMLOG_ERROR()などのマクロ	*/
/* でFLASH保存を再帰的に呼び出す可能性があり危険です	*/
/* 最新の注意を払ってください							*/
/********************************************************/
#if (DCOM_RTT_MODE == 1)
#define LOG_SAVE_ERROR		1		// エラーログをフラッシュに残す場合は、1にする
#define LOG_SAVE_WARN		1		// ワーニングログをフラッシュに残す場合は、1にする
#define LOG_SAVE_DBG		0		// DBGログをフラッシュに残す場合は、1にする
#define LOG_SAVE_INFO		1		// INFOログをフラッシュに残す場合は、1にする
#else	//#if (DCOM_RTT_MODE == 1)
#define LOG_SAVE_ERROR		0		// エラーログをフラッシュに残す場合は、1にする
#define LOG_SAVE_WARN		0		// ワーニングログをフラッシュに残す場合は、1にする
#define LOG_SAVE_DBG		0		// DBGログをフラッシュに残す場合は、1にする
#define LOG_SAVE_INFO		0		// INFOログをフラッシュに残す場合は、1にする
#endif	//#if (DCOM_RTT_MODE == 1)
// CHG-E Y.Hirano 2022/3/25 830POC-49 :GNSS測位時間改善

/* size of stack area used by each thread */
#define STACKSIZE_1K 1024
#define STACKSIZE_2K 2048
#define STACKSIZE_4K 4096
#define STACKSIZE_5K 5120 // ADD K.Saho 2023/03/02 update_threadのスタックサイズを変更
#define STACKSIZE_6K 6144

/* scheduling priority used by each thread */
#define GNSS_PRIORITY 7         // gnss thread
#define LTE_PRIORITY -3         // lte thread
#define UI_PRIORITY 10          // ui thread
#define BUZZER_PRIORITY 10      // buzzer thread
#define SENSOR_PRIORITY 7       // sensor thread
#define MLOCA_PRIORITY 9        // mloca thread
#define WDT_PRIORITY 12         // wdi thread
#define LOGGER_PRIORITY 11      // logger thread
#define UPDATE_PRIORITY 11 			// update thread   // ADD D.Praveen 2022/09/29 FOTAと設定連動用に新規スレッドを作成

// センサースレッドの監視周期
#define SLEEP_TIME_MS 10        // sleep time

// バージョンを意図的に編集する仕組みは不要なので、マスクしておく(必要であれば復活させる)
#if 0
// FWバージョン
// FWメジャーバージョン
#define FW_MAJOR_VER	0
// FWマイナーバージョン
#define FW_MINOR_VER	0
// FWパッチバージョン
#define FW_PATCH_VER	1

// バージョンタイプ数
#define FW_VERSION_NUM	3
// バージョン最大値
#define FW_VERSION_MAX	255
// バージョン最小値
#define FW_VERSION_MIN	0
#endif	// #if 0

// MOV-S M.Taguchi 2023/4/13 830001-112 :設定データ保存箇所とは別セクターでもう一つの出荷検査フラグを管理する
// 検査日時長
#define CHECK_DATE_LEN	16
// MOV-E M.Taguchi 2023/4/13 830001-112 :設定データ保存箇所とは別セクターでもう一つの出荷検査フラグを管理する

// #define LOG_ERROR_FLASH(...)
// macro
// 必要に応じてflashに残す
#if (LOG_SAVE_ERROR == 1)
#define MLOG_ERROR(...)										\
{															\
	LOG_ERR(__VA_ARGS__);									\
	logger_thread_save_mcr(FLASH_LOG_LV_ERR, __VA_ARGS__);	\
}
#else 	// #if (LOG_SAVE_ERROR == 1)
#define MLOG_ERROR(...)									\
{														\
	LOG_ERR(__VA_ARGS__);								\
}
#endif	// #if (LOG_SAVE_ERROR == 1)

// 必要に応じてflashに残す
#if (LOG_SAVE_WARN == 1)
#define MLOG_WARNING(...)										\
{															\
	LOG_WRN(__VA_ARGS__);									\
	logger_thread_save_mcr(FLASH_LOG_LV_WRN, __VA_ARGS__);	\
}
#else 	// #if (LOG_SAVE_WARN == 1)
#define MLOG_WARNING(...)									\
{														\
	LOG_WRN(__VA_ARGS__);								\
}
#endif	// #if (LOG_SAVE_WARN == 1)

// 必要に応じてflashに残す
#if (LOG_SAVE_DBG == 1)
#define MLOG_DEBUG(...)										\
{															\
	LOG_DBG(__VA_ARGS__);									\
	logger_thread_save_mcr(FLASH_LOG_LV_DBG, __VA_ARGS__);	\
}
#else 	// #if (LOG_SAVE_DBG == 1)
#define MLOG_DEBUG(...)									\
{														\
	LOG_DBG(__VA_ARGS__);								\
}
#endif	// #if (LOG_SAVE_DBG == 1)

// 必要に応じてflashに残す
#if (LOG_SAVE_INFO == 1)
#define MLOG_INFO(...)										\
{															\
	LOG_INF(__VA_ARGS__);									\
	logger_thread_save_mcr(FLASH_LOG_LV_INF, __VA_ARGS__);	\
}
#else 	// #if (LOG_SAVE_INFO == 1)
#define MLOG_INFO(...)									\
{														\
	LOG_INF(__VA_ARGS__);								\
}
#endif	// #if (LOG_SAVE_INFO == 1)



// prototype
extern uint16_t calc_CRC( char *Data, int StartAdr, int StopAdr);
// ADD-S Y.Hirano 2022/2/17 830POC-27 :IM830の時間あわせ修正
extern void date_time_setting(char* date_str, int str_len);
extern void date_time_getter_str(char *date, int data_len);		// CHG M.Taguchi 2022/8/29 ビルド時のワーニングエラー削除
extern void data_time_getter_tm(struct tm *ptm);
extern void data_time_getter_tm_JST(struct tm *pget_tm);	// ADD T.Itai 2022/11/30 830001-15 :【FOTA】FOTA更新時間の比較をUTCで行っている
extern void rtt_timestamp_offset_setting(void);
// ADD-E Y.Hirano 2022/2/17 830POC-27 :IM830の時間あわせ修正
// ADD-S M.Taguchi 2022/8/18 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
extern int data_time_convert(struct tm *pget_tm);
extern int diff_date(struct tm *date_time, time_t timer);
#endif // #if (DCOM_AGPS_OFFLINE == 1)
// ADD-E M.Taguchi 2022/8/18 A-GPS AssistNow Offline対応
// ADD-S Y.Hirano 2022/2/21 830POC-34 :LTE通信ができなくなる件の暫定対処
extern void system_reboot_cold(void);
// void system_reboot_warm(void);		// 未使用なのでコメントアウトs
// ADD-E Y.Hirano 2022/2/21 830POC-34 :LTE通信ができなくなる件の暫定対処
// ADD-S D.Praveen 2022/09/26 更新時刻判定APIと更新バージョン判定APIの追加
extern int compare_two_dates_and_time(char* date_time1, char* date_time2);
extern int compare_version(char* version1, char* version2);
// ADD-E D.Praveen 2022/09/26 更新時刻判定APIと更新バージョン判定APIの追加
// ADD-S D.Praveen 2022/10/05 更新時刻判定APIの修正
extern int check_date(int date, int mon, int year);
extern int check_time(int hour, int minute);
extern char *substring(char str[], char substr[], int position, int length);
// ADD-S D.Praveen 2022/10/11 updateスレッドにモデムFOTA処理を追加とアプリFOTA、モデムFOTAの更新ファイルの名前に「_XXX」を付与
extern void removeChar(char* s, char c);
extern int compareStrings(char* string1, char* string2);
// ADD-E D.Praveen 2022/10/11 updateスレッドにモデムFOTA処理を追加とアプリFOTA、モデムFOTAの更新ファイルの名前に「_XXX」を付与
// ADD-E D.Praveen 2022/10/05 更新時刻判定APIの修正
// ADD-S D.Praveen 2022/10/24 s3ログイベントIDsとjson形式を追加
// CHG-S D.Praveen 2022/10/27 static変数の使用を削除
#if 0
extern char* get_s3_log_file_name(char* serial_no);
extern char* get_current_date_YYYY_DD_MM_HH_SS_MM(void);
#else
extern void get_s3_log_file_name(char* serial_no, char log_file_name[]);
extern void get_current_date_YYYY_DD_MM_HH_SS_MM(char current_date[]);
//extern char* join_source_filename_lineno_cause(char* filename, int lineno, char* cause, char cause_string[]);
#endif
// CHG-E D.Praveen 2022/10/27 static変数の使用を削除
// ADD-E D.Praveen 2022/10/24 s3ログイベントIDsとjson形式を追加
// ADD-S T.Itai 2022/10/18 出荷検査対応
extern bool GetMcTestEnableFlg(void);
extern void SetMcTestEnableFlg(bool flg);
extern bool GetMcTestLoopFlg(void);
extern void SetMcTestLoopFlg(bool flg);
// ADD-E T.Itai 2022/10/18 出荷検査対応
extern void unescaped_sequence(char *input, char *output); // ADD K.Saho 2023/02/24 エスケープシーケンス削除対応
extern int 	isDateTimeValid(char* dateTime); 
extern char* increment_year(char* input_date);
#endif //_excommon_H_
/****************************************************************************************/
/* FILE END : ex_common.h																*/
/****************************************************************************************/
