/******************************************************************************/
/*!
 * @file   ex_gnss.h
 * @brief  外部ヘッダー
 * @note
 * @date   作成日　2021年12月16日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/
#ifndef _exgnss_H_
#define _exgnss_H_

#include <zephyr.h>
#include "ex_common.h"
#if (DCOM_AGPS_OFFLINE == 1)
#include <time.h>
#endif // #if (DCOM_AGPS_OFFLINE == 1)
#include "ex_uart.h"


// ADD-S M.Taguchi 2022/8/16 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
#define PAY_ASSDATA_SIZE			64		// assデータサイズ（AGPSデータ）

#define AGPS_RECORD_NUM			    32		// AGPSデータ保存件数/セクター（1セクターには1日最大32レコード分のデータを保存する）
#define AGPS_DAY_MAX			    8		// AGPSデータ取得日数

#define FLASH_SECTOR_AGPS_MANAGE	0x9000  // AGPSデータ管理情報セクターオフセット
#endif // #if (DCOM_AGPS_OFFLINE == 1)
// ADD-E M.Taguchi 2022/8/16 A-GPS AssistNow Offline対応

// GNSS管理情報
typedef struct
{
    uint8_t new_flag;
    uint8_t fix_type;
    uint8_t utc_month;
    uint8_t utc_day;
    uint8_t utc_hour;
    uint8_t utc_min;
    uint8_t utc_sec;
    uint8_t num_sv;
    int32_t longitude;
    int32_t latitude;
    int32_t altitude;
    int32_t g_speed;
    int32_t heading_2d;
    uint32_t h_acc;
    uint32_t v_acc;
    int32_t vel_north;
    int32_t vel_east;
    int32_t vel_down;
    int16_t p_dop;
    int16_t utc_year;
    uint32_t totaldistance;
} GnssInfo;


// ADD-S M.Taguchi 2022/8/16 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
// ペイロード部(AGPSデータ)
typedef struct
{
	unsigned char type;		                    // メッセージタイプ
	unsigned char ver;			                // メッセージバージョン
	unsigned char svid;                         // 衛星ID
	unsigned char gnssid;	                    // GNSSID
	unsigned char year;		                    // 年
	unsigned char month;		                // 月
	unsigned char day;			                // 日
	unsigned char reserved0;					// 予約
	unsigned char assdata[PAY_ASSDATA_SIZE];	// データ
	unsigned char reserved1[4];					// 予約
} AGPS_PAYLOAD;

// AGPSデータ保存用(1レコード)
typedef struct
{
	char header[2];					    // ヘッダー
    char class;					        // クラス
    char id;							// ID
	char length[2];		                // ペイロード長さ
	AGPS_PAYLOAD payload;				// ペイロード
	char checksum[2];				    // チェックサム
} AGPS_RECORD;

// AGPSデータ保存用
typedef struct
{
	AGPS_RECORD			record[AGPS_RECORD_NUM * AGPS_DAY_MAX];	    // AGPSデータのレコード
	uint16_t			crc_num;						            // レコードのCRC値（正常性チェックのため）
} AGPS_SECTOR;


// AGPSデータ管理情報
typedef struct
{
#if 0
	unsigned char			year;                           // AGPSダウンロード日時（年）
    unsigned char			month;                          // AGPSダウンロード日時（月）
    unsigned char			day;                            // AGPSダウンロード日時（日）
    unsigned char			hour;                           // AGPSダウンロード日時（時）
    unsigned char			minute;                         // AGPSダウンロード日時（分）
    unsigned char			second;                         // AGPSダウンロード日時（秒）
#endif
    time_t                  epoch_sec;
	uint16_t				record_num[AGPS_DAY_MAX];		// AGPSデータレコード数
} AGPS_MNG;
#endif // #if (DCOM_AGPS_OFFLINE == 1)
// ADD-E M.Taguchi 2022/8/16 A-GPS AssistNow Offline対応


// prototype
// CHG-S M.Taguchi 2022/8/19 A-GPS AssistNow Offline対応
#if (DCOM_AGPS_OFFLINE == 1)
extern void gnss_init(char* pAgpsData, int len, bool use_agps_flg);     // CHG M.Taguchi 2023/2/13 830001-49 :A-GPSデータ格納用ヒープメモリの解放により不正なA-GPSデータがflash書き込みされる
#else	// #if (DCOM_AGPS_OFFLINE == 1)
extern void gnss_init(char* pAgpsData, int len);
#endif	// #if (DCOM_AGPS_OFFLINE == 1)
// CHG-E M.Taguchi 2022/8/19 A-GPS AssistNow Offline対応
extern void gnss_parseUBX(uint8_t *incoming_message, uint32_t incoming_message_length);
extern GnssInfo gnss_info_getter(void);
extern void gnss_uart_msg_push(uart_msg_queue_item *pItem);
// ADD-S Y.Hirano 2022/3/25 830POC-51 :GPSに関連する時間をモバロケへアップする
extern int gnss_get_agps_send_dlt(void);
// ADD-E Y.Hirano 2022/3/25 830POC-51 :GPSに関連する時間をモバロケへアップする
// ADD-S M.Taguchi 2023/5/16 830001-146 :trunk環境のTTFFが遅い
#if (DCOM_TTFF_LOG == 1)
extern void gnss_set_ref_ttff(int64_t ttff);
#endif	// #if (DCOM_TTFF_LOG == 1)
// ADD-E M.Taguchi 2023/5/16 830001-146 :trunk環境のTTFFが遅い

#endif //_exgnss_H_
/****************************************************************************************/
/* FILE END : ex_gnss.h																	*/
/****************************************************************************************/
