/******************************************************************************/
/*!
 * @file   ex_mloca_thread.h
 * @brief  外部ヘッダー
 * @note
 * @date   作成日　2021年12月16日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/
#ifndef _exmlocathread_H_
#define _exmlocathread_H_

#include "ex_common.h"

// レスポンスチェック
#define HTTP_RESPONSE_OK 200
#define HTTP_RESPONSE_NOTFOUND 404

#define MC_API_BUF_SIZE 3072

// CHG M.Taguchi 2023/5/18 830001-148 :kibana送信のセマフォ待ち時間を調整する
#define MLOCA_API_TIMER_DFF 600 // 定期API取得間隔（取得したトークンの有効期限から差し引く秒数）

// HTTPヘッダー関連(可変の項目はキー名のみ)
#define HTTP_HEAD_ACCEPT_ENC "Accept-Encoding: identity" // Accept-Encodeing  →  圧縮指定。等価関数（圧縮変更なし）
#define HTTP_HEAD_CONTENT_LEN "Content-Length: "		 // Content-Length → ボディ部の長さ
#define HTTP_HEAD_ACCEPT "Accept: application/json"		 // Accept → 求めているレスポンスのタイプ
#define HTTP_HEAD_AUTH "Authorization: "				 // Authorization → トークン指定
#define HTTP_HEAD_BEARER "Authorization: Bearer "		 // Authorization → Bearer トークン指定		// ADD M.Taguchi 2023/2/20 830001-48 :設定連動が開始されない
#define HTTP_HEAD_HOST "Host: "							 // Host → ホスト名
#define HTTP_HEAD_CONNECTION "Connection: close" // Connection → レスポンス後のTCP接続を維持するかどうか
#define HTTP_HEAD_CONNENT_TYPE "Content-Type: application/json" // Content-Type → ボディ部の形式

#define API_TMOUT_MAX_RETRY 3 // APIhttpレスポンスタイムアウト最大リトライ回数	// ADD M.Taguchi 2023/3/2 830001-60 :設定連動APIにてhttpタイムアウトだった場合の返り値を判定していない

// #define TOKEN_SIZE_MAX	1024
#define TOKEN_SIZE_MAX 1500

// LicenseAPI
#if (DCOM_STAGING == 1)
#define LICENSE_HOST "staging-license-api.mcapps.jp" // staging
#else // #if (DCOM_STAGING == 1)
#define LICENSE_HOST "ajt-license-api.mcapps.jp"	 // 本番
#endif // #if (DCOM_STAGING == 1)

// クライアントID
#if (DCOM_STAGING == 1)
#define MC_CLIENT_ID "5nf2uabq64m6lf4os2c3vlpjec" // staging
#else // #if (DCOM_STAGING == 1)
#define MC_CLIENT_ID "1d46c4pdk4kkeud09if43vfcb7" // 本番
#endif // #if (DCOM_STAGING == 1)

// enun
// スレッドステータス
// ADD-S D.Praveen 2023/05/24 830001-158 :モバロケスレッドの定期実行中に10秒毎の時間発行が行われない
typedef enum
{
	MLOCA_THREAD_STS_INIT = 0,		// 初期状態
	MLOCA_THREAD_STS_IDLE,			// アイドル状態
	MLOCA_THREAD_STS_DISCONNECT,	// 切断状態
	MLOCA_THREAD_STS_NUM,			// 最大値
} MLOCA_THREAD_STATUS;
// ADD-E D.Praveen 2023/05/24 830001-158 :モバロケスレッドの定期実行中に10秒毎の時間発行が行われない

// prototype
// CHG-S T.Itai 2022/7/28
#if 0
extern void mloca_thread_create(void);
#else
extern int mloca_thread_create(void);
#endif
// CHG-E T.Itai 2022/7/28
extern void mloca_thread_start(void);
extern void mloca_thread_lte_connect(void);
extern void mloca_thread_lte_disconnect(void);
extern void mlloca_thread_issue(int evt_type);

// ADD-S T.Itai 2022/09/14 FOTA用JSONをS3から取得
extern int mc_api_call(char *pReqMsg, int Msglen, char *pHost, int HostLen, char *pRcvBuf, int RcvBuflen);
//extern int mc_http_api_call(char *pReqMsg, int Msglen, char *pHost, int HostLen, char *pRcvBuf, int RcvBuflen);
extern int mc_api_call2(char *pReqMsg, int Msglen, char *pHost, int HostLen, char *pRcvBuf, int RcvBuflen);
extern int mc_api_check_response(char *pRes, int len);
// ADD-E T.Itai 2022/09/14 FOTA用JSONをS3から取得

extern void mloca_thread_auth_result(int auth_return);      // ADD M.Taguchi 2023/05/23 830001-158 :モバロケスレッドの定期実行中に10秒毎の時間発行が行われない
extern int mloca_thread_status_get(void);					// ADD D.Praveen 2023/05/24 830001-158 :モバロケスレッドの定期実行中に10秒毎の時間発行が行われない
extern int mc_api_preodic_exec(void); 						// ADD D.Praveen 2023/05/24 830001-158 :モバロケスレッドの定期実行中に10秒毎の時間発行が行われない
#endif //_exmlocathread_H_
/****************************************************************************************/
/* FILE END : ex_mloca_thread.h															*/
/****************************************************************************************/
